<?php

/**
 * @file
 * Contains Drupal\pylot_bridge\Plugin\Filter.
 */

namespace Drupal\pylot_bridge\Plugin\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\pylot_bridge\Services\BridgeShortCodeParser;
use Drupal\pylot_bridge\Services\BridgeUtils;

/**
 * Provides a filter to replace  with code wrappers.
 *
 * @Filter(
 *  id = "bridge_filter",
 *  module = "pylot_bridge",
 *  title = @Translation("Pylot Bridge : shortcodes"),
 *  type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE,
 *  cache = FALSE,
 *  weight = 0
 * )
 */
class BridgeFilter extends FilterBase {

    private $attachListLibraries = false;
    private $attachFicheLibraries = false;

    /**
     * {@inheritdoc}
     */
    public function process($text, $langcode) {
        $new_text = $text;

        $shortcodes = BridgeShortCodeParser::parse_shortcodes($text);

        // Shortcode [brliste id="XX"]
        $new_text = $this->processWeblists($new_text, $shortcodes, $langcode);
        $new_text = $this->processWebPages($new_text, $shortcodes, $langcode);

        $result = new FilterProcessResult($new_text);
        if($this->attachListLibraries) {
            $libraries = BridgeUtils::getListLibraries();
            $result->setAttachments(array(
                'library' => $libraries,
            ));
        }
        if($this->attachFicheLibraries) {
            $libraries = BridgeUtils::getFicheLibraries();
            $result->setAttachments(array(
                'library' => $libraries,
            ));
        }
        return $result;
    }

    /**
     * Traite le shortcode brliste
     * @param $text
     * @param $langcode
     * @return array|string|string[]
     */
    private function processWeblists($text, $shortcodes, $langcode) {
        $new_text = $text;
        if(is_array($shortcodes) && count($shortcodes) > 0 ) {
            foreach($shortcodes as $shortcode) {
                if($shortcode['name'] !== 'brliste')
                    continue;
                $attributes = array();
                if(empty($shortcode['attrs']))
                    continue;
                foreach($shortcode['attrs'] as $attr) {
                    foreach($attr as $key => $value) {
                        $attributes[$key] = str_replace('"', '', $value);
                    }
                }
                if(!isset($attributes['id']))
                    continue;
                $attributes['lang'] = $langcode;
                $attributes['change'] = '1';
                $attributes['useRequestFilters'] = false;
                try {
                    // Appel à Bridge, récupération des données
                    $data = BridgeUtils::getListData($attributes);
                    // On prépare l'organisation des données en vue de l'affichage - avec conversion des tableaux associatifs en objets au passage pour simplifier
                    $data = BridgeUtils::prepareListDataForRender($data);
                    // On attache les librairies scripts, css ainsi que le tableau de données à passer aux templates d'affichage
                    $build = BridgeUtils::prepareListViewRenderer($data, 'liste_sit_shortcode');
                    // On fait le rendu HTML
                    $html = \Drupal::service('renderer')->renderPlain($build);
                    // On avertit le parent qu'il faut mettre les librairies
                    $this->attachListLibraries = true;
                    $new_text = str_replace($shortcode['shortcode'], $html, $new_text);

                } catch (\Exception $e) {
                    $html =  "Erreur 07 shortcode Bridge : " . $e->getMessage() ;
                    $new_text = str_replace($shortcode['shortcode'], $html, $new_text);
                }
            }
        }
        return $new_text;
    }

    /**
     * Traite le shortcode brfiche
     * @param $text
     * @param $langcode
     * @return array|string|string[]
     */
    private function processWebPages($text, $shortcodes, $langcode) {
        $new_text = $text;
        if(is_array($shortcodes) && count($shortcodes) > 0 ) {
            foreach($shortcodes as $shortcode) {
                if($shortcode['name'] !== 'brfiche') {
                    continue;

                }
                $attributes = array();
                if(empty($shortcode['attrs'])) {
                    continue;
                }

                foreach($shortcode['attrs'] as $attr) {
                    foreach($attr as $key => $value) {
                        $attributes[$key] = str_replace('"', '', $value);
                    }
                }
                if(!isset($attributes['id']) || !isset($attributes['product_code'])) {
                    continue;
                }

                $attributes['lang'] = $langcode;
                try {
                    // Récupération des données de la fiche SIT
                    $data = BridgeUtils::getDataFiche($attributes['product_code'], $attributes['id']);
                    if($data === false) {
                        $html =  "Erreur 08 shortcode Bridge : ";
                        $new_text = str_replace($shortcode['shortcode'], $html, $new_text);
                    } else {
                        // On prepare la vue
                        $build = BridgeUtils::prepareFicheViewRenderer($data, 'fiche_sit_ajax');

                        // On fait le rendu HTML
                        $html = \Drupal::service('renderer')->renderPlain($build);
                        // On avertit le parent qu'il faut mettre les librairies
                        $this->attachFicheLibraries = true;
                        $new_text = str_replace($shortcode['shortcode'], $html, $new_text);
                    }


                } catch (\Exception $e) {
                    $html =  "Erreur 071 shortcode Bridge : " . $e->getMessage();
                    $new_text = str_replace($shortcode['shortcode'], $html, $new_text);
                }
            }
        } else {

        }
        return $new_text;
    }

}